@@ -12,6 +12,7 @@
 
 import atexit
 import os
+import threading
 
 import glance_store
 from oslo_config import cfg
@@ -22,6 +23,7 @@ from glance.api import common
 import glance.async_
 from glance.common import config
 from glance.common import store_utils
+from glance import housekeeping
 from glance.i18n import _
 from glance import notifier
 
@@ -79,6 +81,17 @@ def drain_threadpools():
         pool_model.pool.shutdown()
 
 
+def run_staging_cleanup():
+    cleaner = housekeeping.StagingStoreCleaner(glance.db.get_api())
+    # NOTE(danms): Start thread as a daemon. It is still a
+    # single-shot, but this will not block our shutdown if it is
+    # running.
+    cleanup_thread = threading.Thread(
+        target=cleaner.clean_orphaned_staging_residue,
+        daemon=True)
+    cleanup_thread.start()
+
+
 def init_app():
     config.set_config_defaults()
     config_files = _get_config_files()
@@ -111,5 +124,7 @@ def init_app():
         glance_store.create_stores(CONF)
         glance_store.verify_default_store()
 
+    run_staging_cleanup()
+
     _setup_os_profiler()
     return config.load_paste_app('glance-api')
