@@ -16,8 +16,6 @@
 """
 Cache driver that uses SQLite to store information about cached images
 """
-
-from __future__ import absolute_import
 from contextlib import contextmanager
 import os
 import sqlite3
@@ -26,9 +24,11 @@ import time
 
 from eventlet import sleep
 from eventlet import timeout
+from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
+from oslo_utils import fileutils
 
 from glance.common import exception
 from glance.i18n import _, _LE, _LI, _LW
@@ -119,26 +119,32 @@ class Driver(base.Driver):
     def initialize_db(self):
         db = CONF.image_cache_sqlite_db
         self.db_path = os.path.join(self.base_dir, db)
-        try:
-            conn = sqlite3.connect(self.db_path, check_same_thread=False,
-                                   factory=SqliteConnection)
-            conn.executescript("""
-                CREATE TABLE IF NOT EXISTS cached_images (
-                    image_id TEXT PRIMARY KEY,
-                    last_accessed REAL DEFAULT 0.0,
-                    last_modified REAL DEFAULT 0.0,
-                    size INTEGER DEFAULT 0,
-                    hits INTEGER DEFAULT 0,
-                    checksum TEXT
-                );
-            """)
-            conn.close()
-        except sqlite3.DatabaseError as e:
-            msg = _("Failed to initialize the image cache database. "
-                    "Got error: %s") % e
-            LOG.error(msg)
-            raise exception.BadDriverConfiguration(driver_name='sqlite',
-                                                   reason=msg)
+        lockutils.set_defaults(self.base_dir)
+
+        @lockutils.synchronized('image_cache_db_init', external=True)
+        def create_db():
+            try:
+                conn = sqlite3.connect(self.db_path, check_same_thread=False,
+                                       factory=SqliteConnection)
+                conn.executescript("""
+                    CREATE TABLE IF NOT EXISTS cached_images (
+                        image_id TEXT PRIMARY KEY,
+                        last_accessed REAL DEFAULT 0.0,
+                        last_modified REAL DEFAULT 0.0,
+                        size INTEGER DEFAULT 0,
+                        hits INTEGER DEFAULT 0,
+                        checksum TEXT
+                    );
+                """)
+                conn.close()
+            except sqlite3.DatabaseError as e:
+                msg = _("Failed to initialize the image cache database. "
+                        "Got error: %s") % e
+                LOG.error(msg)
+                raise exception.BadDriverConfiguration(driver_name='sqlite',
+                                                       reason=msg)
+
+        create_db()
 
     def get_cache_size(self):
         """
@@ -253,7 +259,7 @@ class Driver(base.Driver):
         """
         files = [f for f in self.get_cache_files(self.queue_dir)]
         for file in files:
-            os.unlink(file)
+            fileutils.delete_if_exists(file)
         return len(files)
 
     def delete_queued_image(self, image_id):
@@ -263,8 +269,7 @@ class Driver(base.Driver):
         :param image_id: Image ID
         """
         path = self.get_image_filepath(image_id, 'queue')
-        if os.path.exists(path):
-            os.unlink(path)
+        fileutils.delete_if_exists(path)
 
     def clean(self, stall_time=None):
         """
@@ -324,7 +329,8 @@ class Driver(base.Driver):
 
                 # Make sure that we "pop" the image from the queue...
                 if self.is_queued(image_id):
-                    os.unlink(self.get_image_filepath(image_id, 'queue'))
+                    fileutils.delete_if_exists(
+                        self.get_image_filepath(image_id, 'queue'))
 
                 filesize = os.path.getsize(final_path)
                 now = time.time()
@@ -446,7 +452,7 @@ class Driver(base.Driver):
         Removes any invalid cache entries
         """
         for path in self.get_cache_files(self.invalid_dir):
-            os.unlink(path)
+            fileutils.delete_if_exists(path)
             LOG.info(_LI("Removed invalid cache file %s"), path)
 
     def delete_stalled_files(self, older_than):
@@ -460,7 +466,7 @@ class Driver(base.Driver):
         for path in self.get_cache_files(self.incomplete_dir):
             if os.path.getmtime(path) < older_than:
                 try:
-                    os.unlink(path)
+                    fileutils.delete_if_exists(path)
                     LOG.info(_LI("Removed stalled cache file %s"), path)
                 except Exception as e:
                     msg = (_LW("Failed to delete file %(path)s. "
@@ -496,9 +502,5 @@ class Driver(base.Driver):
 
 
 def delete_cached_file(path):
-    if os.path.exists(path):
-        LOG.debug("Deleting image cache file '%s'", path)
-        os.unlink(path)
-    else:
-        LOG.warn(_LW("Cached image file '%s' doesn't exist, unable to"
-                     " delete") % path)
+    LOG.debug("Deleting image cache file '%s'", path)
+    fileutils.delete_if_exists(path)
