@@ -14,6 +14,7 @@
 #    under the License.
 
 import os
+from unittest import mock
 
 import glance_store as store
 from glance_store import location
@@ -39,8 +40,7 @@ class StoreClearingUnitTest(test_utils.BaseTestCase):
         self.addCleanup(setattr, location, 'SCHEME_TO_CLS_MAP', dict())
 
     def _create_stores(self, passing_config=True):
-        """Create known stores. Mock out sheepdog's subprocess dependency
-        on collie.
+        """Create known stores.
 
         :param passing_config: making store driver passes basic configurations.
         :returns: the number of how many store drivers been loaded.
@@ -65,20 +65,23 @@ class MultiStoreClearingUnitTest(test_utils.BaseTestCase):
         self.addCleanup(setattr, location, 'SCHEME_TO_CLS_MAP', dict())
 
     def _create_multi_stores(self, passing_config=True):
-        """Create known stores. Mock out sheepdog's subprocess dependency
-        on collie.
+        """Create known stores.
 
         :param passing_config: making store driver passes basic configurations.
         :returns: the number of how many store drivers been loaded.
         """
-        self.config(enabled_backends={'file1': 'file', 'ceph1': 'rbd'})
+        self.config(enabled_backends={'fast': 'file', 'cheap': 'file',
+                                      'readonly_store': 'http',
+                                      'fast-cinder': 'cinder'})
         store.register_store_opts(CONF)
 
-        self.config(default_backend='file1',
+        self.config(default_backend='fast',
                     group='glance_store')
 
         self.config(filesystem_store_datadir=self.test_dir,
-                    group='file1')
+                    group='fast')
+        self.config(filesystem_store_datadir=self.test_dir2,
+                    group='cheap')
         store.create_multi_stores(CONF)
 
 
@@ -88,7 +91,6 @@ class IsolatedUnitTest(StoreClearingUnitTest):
     Unit test case that establishes a mock environment within
     a testing directory (in isolation)
     """
-    registry = None
 
     def setUp(self):
         super(IsolatedUnitTest, self).setUp()
@@ -102,9 +104,18 @@ class IsolatedUnitTest(StoreClearingUnitTest):
                     group="glance_store")
 
         store.create_stores()
-        stubs.stub_out_registry_and_store_server(self.stubs,
-                                                 self.test_dir,
-                                                 registry=self.registry)
+
+        def fake_get_conection_type(client):
+            DEFAULT_API_PORT = 9292
+
+            if client.port == DEFAULT_API_PORT:
+                return stubs.FakeGlanceConnection
+
+        self.patcher = mock.patch(
+            'glance.common.client.BaseClient.get_connection_type',
+            fake_get_conection_type)
+        self.addCleanup(self.patcher.stop)
+        self.patcher.start()
 
     def set_policy_rules(self, rules):
         fap = open(CONF.oslo_policy.policy_file, 'w')
@@ -118,7 +129,6 @@ class MultiIsolatedUnitTest(MultiStoreClearingUnitTest):
     Unit test case that establishes a mock environment within
     a testing directory (in isolation)
     """
-    registry = None
 
     def setUp(self):
         super(MultiIsolatedUnitTest, self).setUp()
@@ -126,11 +136,19 @@ class MultiIsolatedUnitTest(MultiStoreClearingUnitTest):
         lockutils.set_defaults(os.path.join(self.test_dir))
 
         self.config(debug=False)
-        stubs.stub_out_registry_and_store_server(self.stubs,
-                                                 self.test_dir,
-                                                 registry=self.registry)
 
     def set_policy_rules(self, rules):
         fap = open(CONF.oslo_policy.policy_file, 'w')
         fap.write(jsonutils.dumps(rules))
         fap.close()
+
+    def mock_object(self, obj, attr_name, *args, **kwargs):
+        """Use python mock to mock an object attribute
+
+        Mocks the specified objects attribute with the given value.
+        Automatically performs 'addCleanup' for the mock.
+        """
+        patcher = mock.patch.object(obj, attr_name, *args, **kwargs)
+        result = patcher.start()
+        self.addCleanup(patcher.stop)
+        return result
