@@ -15,9 +15,9 @@
 #    under the License.
 
 import datetime
+from unittest import mock
 import uuid
 
-import mock
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_utils import encodeutils
@@ -35,32 +35,6 @@ CONF = cfg.CONF
 CONF.import_opt('metadata_encryption_key', 'glance.common.config')
 
 
-@mock.patch('oslo_utils.importutils.import_module')
-class TestDbUtilities(test_utils.BaseTestCase):
-    def setUp(self):
-        super(TestDbUtilities, self).setUp()
-        self.config(data_api='silly pants')
-        self.api = mock.Mock()
-
-    def test_get_api_calls_configure_if_present(self, import_module):
-        import_module.return_value = self.api
-        self.assertEqual(glance.db.get_api(), self.api)
-        import_module.assert_called_once_with('silly pants')
-        self.api.configure.assert_called_once_with()
-
-    def test_get_api_skips_configure_if_missing(self, import_module):
-        import_module.return_value = self.api
-        del self.api.configure
-        self.assertEqual(glance.db.get_api(), self.api)
-        import_module.assert_called_once_with('silly pants')
-        self.assertFalse(hasattr(self.api, 'configure'))
-
-    def test_get_api_calls_for_v1_api(self, import_module):
-        api = glance.db.get_api(v1_mode=True)
-        self.assertNotEqual(api, self.api)
-        import_module.assert_called_once_with('glance.db.sqlalchemy.api')
-        api.configure.assert_called_once_with()
-
 UUID1 = 'c80a1a6c-bd1f-41c5-90ee-81afedb1d58d'
 UUID2 = 'a85abd86-55b3-4d5b-b0b4-5d0a6e6042fc'
 UUID3 = '971ec09a-8067-4bc8-a91f-ae3557f1c4c7'
@@ -394,6 +368,44 @@ class TestImageRepo(test_utils.BaseTestCase):
                                 image)
         self.assertIn(fake_uuid, encodeutils.exception_to_unicode(exc))
 
+    def test_save_excludes_atomic_props(self):
+        fake_uuid = str(uuid.uuid4())
+        image = self.image_repo.get(UUID1)
+
+        # Try to set the property normally
+        image.extra_properties['os_glance_import_task'] = fake_uuid
+        self.image_repo.save(image)
+
+        # Expect it was ignored
+        image = self.image_repo.get(UUID1)
+        self.assertNotIn('os_glance_import_task', image.extra_properties)
+
+        # Set the property atomically
+        self.image_repo.set_property_atomic(image,
+                                            'os_glance_import_task', fake_uuid)
+        # Expect it is set
+        image = self.image_repo.get(UUID1)
+        self.assertEqual(fake_uuid,
+                         image.extra_properties['os_glance_import_task'])
+
+        # Try to clobber it
+        image.extra_properties['os_glance_import_task'] = 'foo'
+        self.image_repo.save(image)
+
+        # Expect it is unchanged
+        image = self.image_repo.get(UUID1)
+        self.assertEqual(fake_uuid,
+                         image.extra_properties['os_glance_import_task'])
+
+        # Try to delete it
+        del image.extra_properties['os_glance_import_task']
+        self.image_repo.save(image)
+
+        # Expect it is still present and set accordingly
+        image = self.image_repo.get(UUID1)
+        self.assertEqual(fake_uuid,
+                         image.extra_properties['os_glance_import_task'])
+
     def test_remove_image(self):
         image = self.image_repo.get(UUID1)
         previous_update_time = image.updated_at
@@ -436,6 +448,47 @@ class TestImageRepo(test_utils.BaseTestCase):
                           self.context,
                           image_id)
 
+    def test_image_set_property_atomic(self):
+        image_id = uuid.uuid4()
+        image = _db_fixture(image_id, name='test')
+
+        self.assertRaises(exception.ImageNotFound,
+                          self.db.image_set_property_atomic,
+                          image_id, 'foo', 'bar')
+
+        self.db.image_create(self.context, image)
+        self.db.image_set_property_atomic(image_id, 'foo', 'bar')
+        image = self.db.image_get(self.context, image_id)
+        self.assertEqual('foo', image['properties'][0]['name'])
+        self.assertEqual('bar', image['properties'][0]['value'])
+
+    def test_set_property_atomic(self):
+        image = self.image_repo.get(UUID1)
+        self.image_repo.set_property_atomic(image, 'foo', 'bar')
+        image = self.image_repo.get(image.image_id)
+        self.assertEqual({'foo': 'bar'}, image.extra_properties)
+
+    def test_image_delete_property_atomic(self):
+        image_id = uuid.uuid4()
+        image = _db_fixture(image_id, name='test')
+
+        self.assertRaises(exception.NotFound,
+                          self.db.image_delete_property_atomic,
+                          image_id, 'foo', 'bar')
+        self.db.image_create(self.context, image)
+        self.db.image_set_property_atomic(image_id, 'foo', 'bar')
+        self.db.image_delete_property_atomic(image_id, 'foo', 'bar')
+        image = self.image_repo.get(image_id)
+        self.assertEqual({}, image.extra_properties)
+
+    def test_delete_property_atomic(self):
+        image = self.image_repo.get(UUID1)
+        self.image_repo.set_property_atomic(image, 'foo', 'bar')
+        image = self.image_repo.get(image.image_id)
+        self.image_repo.delete_property_atomic(image, 'foo', 'bar')
+        image = self.image_repo.get(image.image_id)
+        self.assertEqual({}, image.extra_properties)
+
 
 class TestEncryptedLocations(test_utils.BaseTestCase):
     def setUp(self):
