@@ -16,7 +16,8 @@
 
 from cryptography import exceptions as crypto_exception
 import glance_store as store
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 from six.moves import urllib
 
@@ -65,7 +66,7 @@ def sort_url_by_qs_keys(url):
 def get_fake_request(path='', method='POST', is_admin=False, user=USER1,
                      roles=None, tenant=TENANT1):
     if roles is None:
-            roles = ['member']
+        roles = ['member']
 
     req = wsgi.Request.blank(path)
     req.method = method
@@ -230,9 +231,9 @@ class FakeStoreAPI(object):
             size = len(data.data.fd)
         if (current_store_size + size) > store_max_size:
             raise exception.StorageFull()
-        if context.user == USER2:
+        if context.user_id == USER2:
             raise exception.Forbidden()
-        if context.user == USER3:
+        if context.user_id == USER3:
             raise exception.StorageWriteDenied()
         self.data[image_id] = (data, size)
         checksum = 'Z'
@@ -253,9 +254,9 @@ class FakeStoreAPI(object):
             size = len(data.data.fd)
         if (current_store_size + size) > store_max_size:
             raise exception.StorageFull()
-        if context.user == USER2:
+        if context.user_id == USER2:
             raise exception.Forbidden()
-        if context.user == USER3:
+        if context.user_id == USER3:
             raise exception.StorageWriteDenied()
         self.data[image_id] = (data, size)
         checksum = 'Z'
@@ -269,6 +270,20 @@ class FakeStoreAPI(object):
         pass
 
 
+class FakeStoreAPIReader(FakeStoreAPI):
+    """A store API that actually reads from the data pipe."""
+
+    def add_to_backend_with_multihash(self, conf, image_id, data, size,
+                                      hashing_algo, scheme=None, context=None,
+                                      verifier=None):
+        for chunk in data:
+            pass
+
+        return super(FakeStoreAPIReader, self).add_to_backend_with_multihash(
+            conf, image_id, data, size, hashing_algo,
+            scheme=scheme, context=context, verifier=verifier)
+
+
 class FakePolicyEnforcer(object):
     def __init__(self, *_args, **kwargs):
         self.rules = {}
